==> punes-0.102.tar.gz: puNES-0.102/src/core/mappers/mapper_BMCG146.c <==
/*
 *  Copyright (C) 2010-2017 Fabio Cavallo (aka FHorse)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include "mappers.h"
#include "info.h"
#include "mem_map.h"

void map_init_BMCG146(void) {
	BYTE i;

	EXTCL_CPU_WR_MEM(BMCG146);

	for (i = 0; i < 4; i++) {
		mapper.rom_map_to[i] = i;
	}
}
void extcl_cpu_wr_mem_BMCG146(WORD address, BYTE value) {
	if (address & 0x0800) {
		value = (address & 0x1F) | (address & ((address & 0x40) >> 6));
		control_bank(info.prg.rom[0].max.banks_16k)
		map_prg_rom_8k(2, 0, value);

		value = (address & 0x18) | 0x07;
		control_bank(info.prg.rom[0].max.banks_16k)
		map_prg_rom_8k(2, 2, value);
	} else {
		if (address & 0x40) {
			value = address & 0x1F;
			control_bank(info.prg.rom[0].max.banks_16k)
			map_prg_rom_8k(2, 0, value);
			map_prg_rom_8k(2, 2, value);
		} else {
			value = (address >> 1) & 0x0F;
			control_bank(info.prg.rom[0].max.banks_32k)
			map_prg_rom_8k(4, 0, value);
		}
	}
	map_prg_rom_8k_update();

	if (address & 0x0080) {
		mirroring_H();
	} else  {
		mirroring_V();
	}
}

==> punes-0.110.tar.gz: puNES-0.110/src/core/mappers/mapper_BMCG146.c <==
/*
 *  Copyright (C) 2010-2023 Fabio Cavallo (aka FHorse)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include "mappers.h"
#include "info.h"
#include "mem_map.h"

void map_init_BMCG146(void) {
	BYTE i;

	EXTCL_CPU_WR_MEM(BMCG146);

	for (i = 0; i < 4; i++) {
		mapper.rom_map_to[i] = i;
	}
}
void extcl_cpu_wr_mem_BMCG146(WORD address, BYTE value) {
	if (address & 0x0800) {
		value = (address & 0x1F) | (address & ((address & 0x40) >> 6));
		control_bank(info.prg.rom.max.banks_16k)
		map_prg_rom_8k(2, 0, value);

		value = (address & 0x18) | 0x07;
		control_bank(info.prg.rom.max.banks_16k)
		map_prg_rom_8k(2, 2, value);
	} else {
		if (address & 0x40) {
			value = address & 0x1F;
			control_bank(info.prg.rom.max.banks_16k)
			map_prg_rom_8k(2, 0, value);
			map_prg_rom_8k(2, 2, value);
		} else {
			value = (address >> 1) & 0x0F;
			control_bank(info.prg.rom.max.banks_32k)
			map_prg_rom_8k(4, 0, value);
		}
	}
	map_prg_rom_8k_update();

	if (address & 0x0080) {
		mirroring_H();
	} else  {
		mirroring_V();
	}
}
